from .list_executions import add_subparser as add_subparser_list_executions
from .list_state_machines import add_subparser as add_subparser_list_state_machines
from .describe_state_machine import add_subparser as add_subparser_describe_state_machine
from .state_machine_graph import add_subparser as add_subparser_state_machine_graph



import functools
import glob
from typing import Callable, Optional

import pydantic.alias_generators


@functools.cache
def get_icons() -> list[str]:
    return glob.glob('asset/aws/*.svg')


def aws_icon(name: str) -> Optional[str]:
    icons = get_icons()
    mappings = {
        'dynamodb': 'DynamoDB',
        'states': 'Step-Functions',
    }

    name = mappings.get(name, name)

    fns: list[Callable[[str], str]] = [
        lambda name: name,
        lambda name: pydantic.alias_generators.to_pascal(name),
    ]

    for fn in fns:
        elm = fn(name)
        if (res := f'asset/aws/Arch_{elm}_16.svg') in icons:
            return res

        if (res := f'asset/aws/Arch_Amazon-{elm}_16.svg') in icons:
            return res

        if (res := f'asset/aws/Arch_AWS-{elm}_16.svg') in icons:
            return res



import argparse
import csv
import datetime as dt
from datetime import date, datetime
import sys
from typing import Any, Optional

import boto3


def main(args: argparse.Namespace):
    sfn_client = boto3.client('stepfunctions')
    pagenator = sfn_client.get_paginator('list_executions')
    pages = pagenator.paginate(
        stateMachineArn=args.state_machine_arn,
        PaginationConfig={
            'MaxItems': args.max_items,
        }
    )

    obj: list[dict[str, Any]] = []
    for page in pages:
        for execution in page['executions']:
            obj.append({
                'name': execution['name'],
                'status': execution['status'],
                'start_date': execution['startDate'],
                'stop_date': execution.get('stopDate'),
                'duration': execution.get('stopDate') - execution['startDate'] if execution.get('stopDate') else '',
            })

    writer = csv.DictWriter(sys.stdout, obj[0].keys())
    writer.writeheader()
    def fmt(x: Any, timespec: str = 'milliseconds') -> str:
        if isinstance(x, datetime):
            return x.isoformat(timespec=timespec)
        if isinstance(x, date):
            return x.isoformat()
        if isinstance(x, dt.timedelta):
            x_mod = x - dt.timedelta(microseconds=x.microseconds)
            return str(x_mod)
        if x is None:
            return ''
        return str(x)

    for row_ in obj:
        row = {
            k: fmt(v, timespec='seconds')
            for k, v in row_.items()
        }
        writer.writerow(row)


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore
    parser = subparsers.add_parser('list-executions')
    parser.add_argument('--state-machine-arn', required=True)
    parser.add_argument('--max-items', type=int, default=100)
    parser.set_defaults(handler=main)



import argparse
import inspect

from . import cmd


def parse_args() -> tuple[argparse.ArgumentParser, argparse.Namespace]:
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    add_subparsers = [
        fn
        for name, fn in inspect.getmembers(cmd, inspect.isfunction)
        if name.startswith('add_subparser_')
    ]

    for add_subparser in add_subparsers:
        add_subparser(subparsers)

    return parser, parser.parse_args()

def main() -> None:
    parser, args = parse_args()

    if hasattr(args, 'handler'):
        args.handler(args)
        return

    parser.print_help()



import argparse
import csv
import datetime as dt
from datetime import date, datetime
import json
import re
import sys
from typing import Any, Optional

import boto3

from .. import util


def main(args: argparse.Namespace):
    obj = json.loads(sys.stdin.read())

    print('digraph {')
    print('node [shape=box, margin="0.3,0"];')
    print('node [fontname="Helvetica,Arial,sans-serif"];')
    print('edge [fontname="Helvetica,Arial,sans-serif"];')
    print('START [shape=ellipse];')
    print('END [shape=doublecircle, margin=0.05];')
    print(f'START -> {obj["StartAt"]}')

    for state_name, state_body in obj['States'].items():
        regexp = r'arn:aws:states:::(?P<service>[^:]+):(?P<action>[^:]+)'
        if (m := re.match(regexp, state_body.get('Resource', ''))):
            service = m.group('service')
            action = m.group('action')
            print(f'{state_name} [label=<<FONT POINT-SIZE="10" COLOR="gray40">{service}.{action}</FONT><BR />{state_name}>]')
            if icon := util.asset.aws_icon.aws_icon(service):
                print(f'{state_name} [image="{icon}", imagepos="tl"]')

        else:
            print(f'{state_name} [label=<<FONT POINT-SIZE="10" COLOR="gray40">{state_body["Type"]}</FONT><BR />{state_name}>]')

        if state_body['Type'] == 'Pass':
            pass

        elif state_body['Type'] == 'Task':
            pass

        elif state_body['Type'] == 'Choice':
            for choice in state_body['Choices']:
                print(f'{state_name} -> {choice["Next"]}')
            if state_body.get('Default'):
                print(f'{state_name} -> {state_body["Default"]}[style="bold"]')

        elif state_body['Type'] == 'Wait':
            pass

        elif state_body['Type'] == 'Succeed':
            print(f'{state_name} -> END')

        elif state_body['Type'] == 'Fail':
            pass

        elif state_body['Type'] == 'Parallel':
            pass

        elif state_body['Type'] == 'Map':
            pass

        if state_body.get('End'):
            print(f'{state_name} -> END')

        if state_body.get('Next'):
            print(f'{state_name} -> {state_body["Next"]}')

        if state_body.get('Catch'):
            for catch in state_body['Catch']:
                print(f'{state_name} -> {catch["Next"]}[style="dashed"]')

    print('}')


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore
    parser = subparsers.add_parser('state-machine-graph')
    parser.set_defaults(handler=main)



import argparse
import csv
import datetime as dt
from datetime import date, datetime
import json
import sys
from typing import Any, Optional

import boto3


def main(args: argparse.Namespace):
    sfn_client = boto3.client('stepfunctions')
    res = sfn_client.describe_state_machine(
        stateMachineArn=args.state_machine_arn,
    )
    print(json.dumps(res, default=str, indent=2))


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore
    parser = subparsers.add_parser('describe-state-machine')
    parser.add_argument('--state-machine-arn', required=True)
    parser.set_defaults(handler=main)



import argparse

import boto3


def main(args: argparse.Namespace):
    sfn_client = boto3.client('stepfunctions')
    pagenator = sfn_client.get_paginator('list_state_machines')
    pages = pagenator.paginate(
        PaginationConfig={
            'MaxItems': args.max_items,
        }
    )
    for page in pages:
        for state_machine in page['stateMachines']:
            print(state_machine['stateMachineArn'])


def add_subparser(subparsers: argparse._SubParsersAction) -> None:  # type: ignore
    parser = subparsers.add_parser('list-state-machines')
    parser.add_argument('--max-items', type=int, default=100)
    parser.set_defaults(handler=main)
